﻿// -*- C++ -*-
/*!
 * @file  Cnoid_TurtleBot2_io.cpp
 * @brief RTC for operating TurtleBot2 model in choreonoid
 * @date $Date$
 *
 * @author East Japan Accounting Center Co., Ltd
 *
 * (c)2018 The University of Aizu
 * This software is released under the New BSD License.
 *
 * $Id$
 */

#include "Cnoid_TurtleBot2_io.h"
#include <fcntl.h>
#include <fstream>
#include <math.h>
#include <termios.h>
#include <unistd.h>
#include <sys/syscall.h>
#include <sys/types.h>

#include <stdio.h>//test

#include <cmath>

// Module specification
// <rtc-template block="module_spec">
static const char* cnoid_turtlebot2_io_spec[] =
  {
    "implementation_id", "Cnoid_TurtleBot2_io",
    "type_name",         "Cnoid_TurtleBot2_io",
    "description",       "RTC for operating TurtleBot2 model in choreonoid",
    "version",           "1.0.0",
    "vendor",            "EAC",
    "category",          "Choreonoid",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
Cnoid_TurtleBot2_io::Cnoid_TurtleBot2_io(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : BodyIoRTC(manager),
    m_in_odometryIn("in_odometry", m_in_odometry)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
Cnoid_TurtleBot2_io::~Cnoid_TurtleBot2_io()
{
}

RTC::ReturnCode_t Cnoid_TurtleBot2_io::onInitialize(Body* body)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("in_odometry", m_in_odometryIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // </rtc-template>
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t Cnoid_TurtleBot2_io::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Cnoid_TurtleBot2_io::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Cnoid_TurtleBot2_io::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Cnoid_TurtleBot2_io::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Cnoid_TurtleBot2_io::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Cnoid_TurtleBot2_io::onExecute(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Cnoid_TurtleBot2_io::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Cnoid_TurtleBot2_io::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Cnoid_TurtleBot2_io::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Cnoid_TurtleBot2_io::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Cnoid_TurtleBot2_io::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

bool Cnoid_TurtleBot2_io::initializeSimulation(ControllerIO* io)
{
    if(io != NULL)
    {
        Body* body = io->body();
        if(body != NULL)
        {
            // Acquire the choreonoid model link
            m_cnoid_Link[LINK_CHASSIS] = body->link("CHASSIS");
            // Set the actuation mode
            m_cnoid_Link[LINK_CHASSIS]->setActuationMode(Link::LINK_POSITION);
        }
        // get time step
        m_cnoid_TimeStep = io->timeStep();
    }

    return true;
}


void Cnoid_TurtleBot2_io::inputFromSimulator()
{
}


void Cnoid_TurtleBot2_io::outputToSimulator()
{
    if(m_in_odometryIn.isNew())
    {
        m_in_odometryIn.read();
        Vector3 T = Vector3(m_in_odometry.data.position.x, m_in_odometry.data.position.y, 0.2);
        Matrix3 R = rotFromRpy(0.0, 0.0, m_in_odometry.data.heading);
        m_cnoid_Link[LINK_CHASSIS]->setTranslation(T);
        m_cnoid_Link[LINK_CHASSIS]->setRotation(R);
    }
}

void Cnoid_TurtleBot2_io::stopSimulation()
{
}

extern "C"
{
 
  void Cnoid_TurtleBot2_ioInit(RTC::Manager* manager)
  {
    coil::Properties profile(cnoid_turtlebot2_io_spec);
    manager->registerFactory(profile,
                             RTC::Create<Cnoid_TurtleBot2_io>,
                             RTC::Delete<Cnoid_TurtleBot2_io>);
  }
  
};
